      subroutine advem(iord,fld,u,v,scal,scali,dt,fco,fc)
c
c --- version 2.6
      implicit none
c
c combined monotone scheme, for details see section 3.3 (eqs. 34 to 37)
c in smolarkiewicz and clark, 1986, j.comput.phys.,67,no 2, p. 396-438
c and smolarkiewicz and grabowski, 1989, j.comput.phys.
c  fld    - transported mixing ratio, e.g., salinity or temperature
c  u,v    - mass fluxes satisfying continuity equation
c  scal   - spatial increments (squared)
c  scali  - inverse of scal
c  dt     - temporal increment
c  fco,fc - depth of the layer at previous and new time step
c
      include 'dimensions.h'
c
      common/linepr/lp
c
      real fld(idm,jdm),u(idm,jdm),v(idm,jdm),scal(idm),scali(idm),
     .     fco(idm,jdm),fc(idm,jdm)
      common/work/fmx(idm,jdm),fmn(idm,jdm),flp(idm,jdm),fln(idm,jdm),
     .            flx(idm,jdm),fly(idm,jdm),u1(idm,jdm),v1(idm,jdm),
     .            flxdiv(idm,jdm)
      real dt,epsil,tencm,onecm,fmx,fmn,flp,fln,flx,fly,flxdiv,q,cushn,
     .     u1,v1,pp,pn,x
      integer lp,iord,itest,jtest,ip1,im1,jp1,jm1
c
c --- the role of 'epsil' is to avoid division by zero.
c --- if iord=1, scheme reduces to simple donor cell scheme.
      data epsil/1.e-11/,tencm/1.e4/,onecm/1.e3/
      common/testpt/itest,jtest
c
c   positive and negative part operator statement functions
      pp(x)=max(0.,x)
      pn(x)=min(0.,x)
c
c --- compute low-order and part of antidiffusive fluxes
c
      do 2 j=1,jj1
c
      do 22 l=1,isp(j)
      flx(ifp(j,l)  ,j)=0.
      flx(ilp(j,l)+1,j)=0.
c
      do 22 i=ifp(j,l),ilp(j,l)
      im1=max(i-1,ifp(j,l))
      ip1=min(i+1,ilp(j,l))
      fmx(i,j)=max(fld(im1,j),fld(i,j),fld(ip1,j))
   22 fmn(i,j)=min(fld(im1,j),fld(i,j),fld(ip1,j))
c
      do 2 l=1,isu(j)
      do 2 i=ifu(j,l),ilu(j,l)
      u1(i,j)=.5*abs(u(i,j))*(fld(i,j)-fld(i-1,j))
      if (u(i,j).ge.0.) then
        q=fld(i-1,j)
      else
        q=fld(i  ,j)
      end if
    2 flx(i,j)=u(i,j)*q
c
      do 3 i=1,ii1
c
      do 33 l=1,jsp(i)
      fly(i,jfp(i,l)  )=0.
      fly(i,jlp(i,l)+1)=0.
c
      do 33 j=jfp(i,l),jlp(i,l)
      jp1=min(j+1,jlp(i,l))
      jm1=max(j-1,jfp(i,l))
      fmx(i,j)=max(fld(i,jm1),fld(i,jp1),fmx(i,j))
   33 fmn(i,j)=min(fld(i,jm1),fld(i,jp1),fmn(i,j))
c
      do 3 l=1,jsv(i)
      do 3 j=jfv(i,l),jlv(i,l)
      v1(i,j)=.5*abs(v(i,j))*(fld(i,j)-fld(i,j-1))
      if (v(i,j).ge.0.) then
        q=fld(i,j-1)
      else
        q=fld(i,j  )
      end if
    3 fly(i,j)=v(i,j)*q
c
cdiag i=itest
cdiag j=jtest
cdiag write (lp,'('' advem''2i4,f22.3/1pe36.2/0pf18.3,1pe9.2,0pf9.3,
cdiag.1pe9.2,0pf9.3/1pe36.2/0pf36.3)') i,j,fld(i-1,j),u(i,j),fld(i,
cdiag.j-1),v(i,j),fld(i,j),v(i,j+1),fld(i,j+1),u(i+1,j),fld(i+1,j)
c
      do 61 j=1,jj1
      do 61 l=1,isp(j)
      do 61 i=ifp(j,l),ilp(j,l)
      flxdiv(i,j)=(flx(i+1,j)-flx(i,j)+fly(i,j+1)-fly(i,j))*dt*scali(i)
      cushn=.5*(.1*fco(i,j)-fc(i,j))
ccc   cushn=cushn+sqrt(9.*onecm*onecm+cushn*cushn)
      cushn=cushn+sqrt(tencm*tencm+cushn*cushn)
      fld(i,j)=(fld(i,j)*(cushn+fco(i,j))-flxdiv(i,j))
     .                  /(cushn+fc (i,j))
c --- don't let -fld- value go outside range of surrounding values
   61 fld(i,j)=max(fmn(i,j),min(fld(i,j),fmx(i,j)))
c
      if (iord.le.1) go to 100
c
c --- finish computation of antidiffusive fluxes
c
      do 8 j=1,jj1
c
      do 7 l=1,isu(j)
      do 7 i=ifu(j,l),ilu(j,l)
    7 flx(i,j)=u1(i,j)-u(i,j)*(flxdiv(i,j)+flxdiv(i-1,j))
     .   /(fco(i,j)+fco(i-1,j)+fc(i,j)+fc(i-1,j)+epsil)
c
      do 8 l=1,isv(j)
      do 8 i=ifv(j,l),ilv(j,l)
    8 fly(i,j)=v1(i,j)-v(i,j)*(flxdiv(i,j)+flxdiv(i,j-1))
     .   /(fco(i,j)+fco(i,j-1)+fc(i,j)+fc(i,j-1)+epsil)
c
c---- limit antidiffusive fluxes
c
      do 16 j=1,jj1
      do 16 l=1,isp(j)
      do 16 i=ifp(j,l),ilp(j,l)
      flp(i,j)=(fmx(i,j)-fld(i,j))*fc(i,j)*scal(i)/dt / ( epsil
     1  -pn(flx(i+1,j))+pp(flx(i,j))-pn(fly(i,j+1))+pp(fly(i,j))  )
      fln(i,j)=(fld(i,j)-fmn(i,j))*fc(i,j)*scal(i)/dt / ( epsil
     1  +pp(flx(i+1,j))-pn(flx(i,j))+pp(fly(i,j+1))-pn(fly(i,j))  )
   16 continue
c
      do 18 j=1,jj1
c
      do 17 l=1,isu(j)
      do 17 i=ifu(j,l),ilu(j,l)
      flx(i,j)=pp(flx(i,j))*min(1.,flp(i,j),fln(i-1,j))
     .        +pn(flx(i,j))*min(1.,flp(i-1,j),fln(i,j))
   17 continue
c
      do 18 l=1,isv(j)
      do 18 i=ifv(j,l),ilv(j,l)
      fly(i,j)=pp(fly(i,j))*min(1.,flp(i,j),fln(i,j-1))
     .        +pn(fly(i,j))*min(1.,flp(i,j-1),fln(i,j))
   18 continue
c
      do 62 j=1,jj1
      do 62 l=1,isp(j)
      do 62 i=ifp(j,l),ilp(j,l)
      flxdiv(i,j)=(flx(i+1,j)-flx(i,j)+fly(i,j+1)-fly(i,j))*dt*scali(i)
      cushn=.5*(.1*fco(i,j)-fc(i,j))
ccc   cushn=cushn+sqrt(9.*onecm*onecm+cushn*cushn)
      cushn=cushn+sqrt(tencm*tencm+cushn*cushn)
      fld(i,j)=(fld(i,j)*(cushn+fc(i,j))-flxdiv(i,j))
     .                  /(cushn+fc(i,j))
c --- don't let -fld- value go outside range of surrounding values
   62 fld(i,j)=max(fmn(i,j),min(fld(i,j),fmx(i,j)))
c
  100 continue
      return
      end
