      subroutine oldnew(lato,longo,offset)
c
c --- the purpose of oldnew is to find the positions of the old (lat/lon)
c --- grid points in the new (rotated mercator) grid. positions are given
c --- in fractional grid units.
c
c --- the original lat/lon grid points are specified in terms of
c ---      nlato,nlongo  --  grid dimension in lat/lon (i,j) direction,
c ---      grido         --  grid size (degrees),
c ---      xpivo,ypivo   --  the i,j index of the equatorial pivot point.
c
c --- the grid points in the rotated mercator grid are specified in terms of
c ---      nlatn,nlongn  --  grid dimension in (rotated) i,j direction,
c ---      gridn         --  grid size in degrees longitude,
c ---      xpivn,ypivn   --  the i,j index of the equatorial pivot point.
c
c --- in both grids, i (=x) increases to relative south, j (=y) to relative
c --- east.
c ---      offset =  0.0:  mass      points
c ---      offset = -0.5:  vorticity points
c
c ---                       o u t p u t :
c
c ---      xnew,ynew    --  locations of the old grid points in the new
c ---                       (counterclockwise rotated, mercator) grid.
c ---      dxnew,dynew  --  components of unit vector pointing in the old
c ---                       x direction expressed in terms of the new grid
c ---                       directions.
c
      include 'dimensions.h'
      include 'common_blocks.h'
      include 'stmt_functions.h'
c
      if (nlato.ne.lato.or.nlongo.ne.longo) stop '(oldnew)'
c
      gridnr=gridn/radian
      gridor=grido/radian
      do 1 i=1,nlato
      xlato=(xpivo-float(i)-offset)*gridor
      do 1 j=1,nlongo
      xlongo=(float(j)+offset-ypivo)*gridor
      xlatn =-asin(sin(xlongo)*cos(xlato))
      xlongn= asin(sin(xlato)/cos(xlatn))
      xnew(i,j)=xpivn-dist(xlatn,gridn)
      ynew(i,j)=ypivn+xlongn/gridnr
      dxnew(i,j)= sin(xlongo)*sin(xlongn)
 1    dynew(i,j)=-cos(xlongo)/cos(xlatn)
      return
c
c
      entry newold(latn,longn,offset)
c
c --- the purpose of newold is to find the positions of the new (rotated
c --- mercator) grid points in the old (lat/lon) grid. positions are given
c --- in fractional grid units.
c
c ---                       o u t p u t :
c
c ---      xold,yold    --  locations of the new (counterclockwise rotated,
c ---                       mercator) grid points in the old (lat/lon) grid.
c ---      dxold,dyold  --  components of unit vector pointing in the new
c ---                       x direction expressed in terms of the old grid
c ---                       directions.
c
      if (nlatn.ne.latn.or.nlongn.ne.longn) stop '(newold)'
c
      gridnr=gridn/radian
      gridor=grido/radian
      do 2 i=1,nlatn
      xlatn=alat(xpivn-float(i)-offset,gridn)
      do 2 j=1,nlongn
      xlongn=(float(j)+offset-ypivn)*gridnr
      xlato= asin(sin(xlongn)*cos(xlatn))
      xlongo=-asin(sin(xlatn)/cos(xlato))
      xold(i,j)=xpivo-xlato/gridor
      yold(i,j)=ypivo+xlongo/gridor
      dxold(i,j)=sin(xlongo)*sin(xlongn)
 2    dyold(i,j)=cos(xlongo)/cos(xlatn)
      return
      end
