      subroutine bigrid(depth)
c
c --- set loop bounds for irregular basin in c-grid configuration
c --- q,u,v,p are vorticity, u-velocity, v-velocity, and mass points, resp.
c --- 'depth' = basin depth array, zero values indicate land
c
      include 'dimensions.h'
      include 'common_blocks.h'
c
c --- mpd = maximum number of points on a diagonal
      parameter (mpd=jdm)        !  mpd=min(idm,jdm) where allowed by compiler
      dimension depth(idm,jdm)
      dimension id(mpd,idm+jdm),jd(mpd,idm+jdm),npts(idm+jdm)
c
      do 17 i=1,ii
      do 17 j=1,jj
      ip(i,j)=0
      iq(i,j)=0
      iu(i,j)=0
 17   iv(i,j)=0
c
c --- fill single-width inlets
 16   nfill=0
      do 15 j=1,jj1
      ja=2*max0(  1,j-1)-(j-1)
      jb=2*min0(jj1,j+1)-(j+1)
      do 15 i=1,ii1
      ia=2*max0(  1,i-1)-(i-1)
      ib=2*min0(ii1,i+1)-(i+1)
      nzero=0
      if (depth(i,j).gt.0.) then
      if (depth(ia,j).le.0.) nzero=nzero+1
      if (depth(ib,j).le.0.) nzero=nzero+1
      if (depth(i,ja).le.0.) nzero=nzero+1
      if (depth(i,jb).le.0.) nzero=nzero+1
      if (nzero.eq.3) then
      write (*,'('' depth(''i3'',''i3'') has been set to zero'')') i,j
      depth(i,j)=0.
      nfill=nfill+1
      end if
      end if
 15   continue
      if (nfill.gt.0) go to 16
c
c --- mass points are defined where water depth is greater than zero
      do 2 i=1,ii1
      do 2 j=1,jj1
      if (depth(i,j).gt.0.) ip(i,j)=1
  2   continue
c
c --- u,v points are located halfway between any 2 adjoining mass points
      do 3 j=1,jj1
      do 3 i=2,ii1
      if (ip(i-1,j).gt.0.and.ip(i,j).gt.0) iu(i,j)=1
  3   continue
      do 4 j=2,jj1
      do 4 i=1,ii1
      if (ip(i,j-1).gt.0.and.ip(i,j).gt.0) iv(i,j)=1
  4   continue
c
c --- 'interior' q points require water on all 4 sides.
      do 5 j=2,jj1
      do 5 i=2,ii1
      if (min0(ip(i,j),ip(i-1,j),ip(i,j-1),ip(i-1,j-1)).gt.0) iq(i,j)=1
  5   continue
c
c --- determine i and j indices of first (ifd,jfd) and last (ild,jld)
c --- points on each contiguous diagonal section of interior q points.
c --- these interior diagonal indices are required for poisson solver
      do 40 npt=1,mpd
      do 40 ndiag=1,idm+jdm
      id(npt,ndiag)=0
 40   jd(npt,ndiag)=0
      do 41 ndiag=1,idm+jdm
 41   npts(ndiag)=0
      do 42 ndiag=1,idm+jdm
      nsec(ndiag)=0
      do 42 k=1,msd
      ifd(ndiag,k)=0
      jfd(ndiag,k)=0
      ild(ndiag,k)=0
 42   jld(ndiag,k)=0
      ndiag=0
      do 31 i=1,ii
      ndiag=ndiag+1
      i1=i
      j1=1
      npt=0
      k=0
 33   ik=i1-k
      jk=j1+k
      if (ik.lt.1.or.jk.gt.jj) go to 31
      if (iq(ik,jk).ne.1) go to 32
      npt=npt+1
      if (npt.gt.mpd) stop 'bigrid-1'
      id(npt,ndiag)=ik
      jd(npt,ndiag)=jk
      npts(ndiag)=npt
 32   k=k+1
      go to 33
 31   continue
      do 34 j=2,jj
      ndiag=ndiag+1
      i1=ii
      j1=j
      npt=0
      k=0
 36   ik=i1-k
      jk=j1+k
      if (ik.lt.1.or.jk.gt.jj) go to 34
      if (iq(ik,jk).ne.1) go to 35
      npt=npt+1
      if (npt.gt.mpd) stop 'bigrid-2'
      id(npt,ndiag)=ik
      jd(npt,ndiag)=jk
      npts(ndiag)=npt
 35   k=k+1
      go to 36
 34   continue
      do 44 nd=1,ndiag
      if (npts(nd).eq.0) go to 44
      k=1
      npt=1
 47   ifd(nd,k)=id(npt,nd)
      jfd(nd,k)=jd(npt,nd)
 46   npt=npt+1
      if (npt.gt.npts(nd)) go to 45
      if (jd(npt,nd).eq.jd(npt-1,nd)+1) go to 46
      ild(nd,k)=id(npt-1,nd)
      jld(nd,k)=jd(npt-1,nd)
      k=k+1
      if (k.gt.msd) then
      write (*,'('' error in bigrid -- msd too small at i,j ='',2i4,
     .  ''  in diagnonal passing through i,j ='',2i4)')
     .   id(npt,nd),jd(npt,nd),id(1,nd),jd(1,nd)
      stop '(bigrid)'
      end if
      go to 47
 45   ild(nd,k)=id(npt-1,nd)
      jld(nd,k)=jd(npt-1,nd)
      nsec(nd)=k
 44   continue
c
c --- 'promontory' q points require water on 3 (or at least 2 diametrically 
c --- opposed) sides
      do 10 j=2,jj1
      do 10 i=2,ii1
ccc      if (iq(i,j).ne.0) go to 10
ccc      if (ip(i,j).gt.0.and.ip(i-1,j-1).gt.0) iq(i,j)=6
ccc      if (ip(i-1,j).gt.0.and.ip(i,j-1).gt.0) iq(i,j)=7
      if ((ip(i  ,j).gt.0.and.ip(i-1,j-1).gt.0).or.
     .    (ip(i-1,j).gt.0.and.ip(i  ,j-1).gt.0)) iq(i,j)=1
 10   continue
c
c --- determine loop bounds for vorticity points, including interior and
c --- promontory points
      call indxi(iq,ifq,ilq,isq)
      call indxj(iq,jfq,jlq,jsq)
c
c --- vorticity points on borders
c --- 'left' border (j increasing to right)
ccc      do 6 j=1,jj1
ccc      do 6 i=2,ii1
ccc      if (iq(i,j).ne.0) go to 6
ccc      if (ip(i,j).gt.0.and.ip(i-1,j).gt.0) iq(i,j)=2
ccc  6   continue
c --- 'right' border
ccc      do 7 j=2,jj
ccc      do 7 i=2,ii1
ccc      if (iq(i,j).ne.0) go to 7
ccc      if (ip(i,j-1).gt.0.and.ip(i-1,j-1).gt.0) iq(i,j)=3
ccc  7   continue
c --- 'top' border (i increasing downward)
ccc      do 8 j=2,jj1
ccc      do 8 i=1,ii1
ccc      if (iq(i,j).ne.0) go to 8
ccc      if (ip(i,j).gt.0.and.ip(i,j-1).gt.0) iq(i,j)=4
ccc  8   continue
c --- 'bottom' border
ccc      do 9 j=2,jj1
ccc      do 9 i=2,ii
ccc      if (iq(i,j).ne.0) go to 9
ccc      if (ip(i-1,j).gt.0.and.ip(i-1,j-1).gt.0) iq(i,j)=5
ccc  9   continue
c
c --- determine loop indices for mass and velocity points
      call indxi(ip,ifp,ilp,isp)
      call indxj(ip,jfp,jlp,jsp)
      call indxi(iu,ifu,ilu,isu)
      call indxj(iu,jfu,jlu,jsu)
      call indxi(iv,ifv,ilv,isv)
      call indxj(iv,jfv,jlv,jsv)
c
c --- velocity points on borders
c --- 'left' and 'right' border points for v (y pointing right)
ccc      do 13 i=1,ii1
ccc      do 13 j=1,jj1
ccc      if (ip(i,j).ne.1) go to 13
ccc      if (iv(i,j).eq.0) iv(i,j)=2
ccc      if (iv(i,j+1).eq.0) iv(i,j+1)=2
ccc 13   continue
c --- 'top' and 'bottom' border points for u (x pointing down)
ccc      do 14 i=1,ii1
ccc      do 14 j=1,jj1
ccc      if (ip(i,j).ne.1) go to 14
ccc      if (iu(i,j).eq.0) iu(i,j)=2
ccc      if (iu(i+1,j).eq.0) iu(i+1,j)=2
ccc 14   continue
      return
      end
